import java.io.*;
import java.net.*;

public class WAVDecoder
{
	static int RIFF_START = 0;
	static int FORMAT_START = 12;
	
	public static void main(String[] args) throws Exception
	{
		byte[] buffer = new byte[1024];
		String filename = args[0];
		FileInputStream fin = new FileInputStream(filename);
		int amount = fin.read(buffer);
		System.out.println("First chunk: "+amount+" bytes");



		// riff chunk
		int riff = getIntASCII(buffer, 0);
		int totallength = getInt(buffer, 4);
		int wave = getIntASCII(buffer, 8);

		// format chunk
		int fmt = getIntASCII(buffer, FORMAT_START);
		int fmt_length = getInt(buffer, FORMAT_START + 4);
		int always_01 = getShort(buffer, FORMAT_START + 8);
		int channels = getShort(buffer, FORMAT_START + 10);
		int rate = getInt(buffer, FORMAT_START + 12);
		int bytes_per_second = getInt(buffer, FORMAT_START + 16);
		int bytes_per_sample = getShort(buffer, FORMAT_START + 20);
		int bits_per_sample = getShort(buffer, FORMAT_START + 22);

		System.out.println("RIFF          : "+Integer.toHexString(riff));
		System.out.println("Total length  : "+totallength);
		System.out.println("WAVE          : "+Integer.toHexString(wave));

		System.out.println("FMT           : "+Integer.toHexString(fmt));
		System.out.println("Format length : "+fmt_length);
		System.out.println("Always 01?    : "+always_01);
		System.out.println("Channels      : "+channels);
		System.out.println("Rate          : "+rate);
		System.out.println("Bytes/sec     : "+bytes_per_second);
		System.out.println("Bytes/sample  : "+bytes_per_sample);
		System.out.println("Bits/sample   : "+bits_per_sample);

		int DATA_START = FORMAT_START + 8 + fmt_length;

		int data = getIntASCII(buffer, DATA_START);
		int datalength = getInt(buffer, DATA_START + 4);

		System.out.println("DATA          : "+Integer.toHexString(data));
		System.out.println("Data length   : "+datalength);

		addToBuffer(buffer, DATA_START+8, buffer.length - (DATA_START + 8));

		amount = fin.read(buffer);
		while (amount != -1)
		{
			addToBuffer(buffer, 0, amount);
			amount = fin.read(buffer);
		}

		buffer = getBuffer();

		processAudioData(buffer, 0, bytes_per_sample);
				
	}

	static ByteArrayOutputStream baos = new ByteArrayOutputStream();

	static void addToBuffer(byte[] buff, int index, int length)
	{
		baos.write(buff, index, length);
	}

	static byte[] getBuffer()
	{
		return baos.toByteArray();
	}

	static void processAudioData(byte[] buffer, int index, int bytes_per_sample)
	{
		int samplecount = 0;
		while (index < buffer.length)
		{
			if ((samplecount & 0x0F)==0)
				System.out.print("\r\n dc16 ");
			int sample = buffer[index++] & 0xff;
			if (bytes_per_sample==2)
				sample = (sample) | ((buffer[index++] & 0xff) << 8);
			samplecount++;
			System.out.print("0x"+Integer.toHexString(sample));
			if ((samplecount & 0x0F)!=0)
				System.out.print(", ");
		}
	}



	static String hexStr(int b)
	{
		b = b & 0x0ff;
		if (b < 0x10)
			return "0" + Integer.toHexString(b);
		return Integer.toHexString(b);
	}

	static int getInt(byte[] buff, int index)
	{
		//return ((buff[index] & 0x0ff) << 24) | ((buff[index+1] & 0x0ff) << 16) | ((buff[index+2] & 0x0ff) << 8) | (buff[index+3] & 0x0ff);
		return (buff[index] & 0x0ff) | ((buff[index+1] & 0x0ff) << 8) | ((buff[index+2] & 0x0ff) << 16) | ((buff[index+3] & 0x0ff) << 24);
	}

	static int getIntASCII(byte[] buff, int index)
	{
		return ((buff[index] & 0x0ff) << 24) | ((buff[index+1] & 0x0ff) << 16) | ((buff[index+2] & 0x0ff) << 8) | (buff[index+3] & 0x0ff);
	}

	static int getShort(byte[] buff, int index)
	{
		//return ((buff[index] & 0x0ff) << 8) | (buff[index+1] & 0x0ff);
		return (buff[index] & 0x0ff) | ((buff[index+1] & 0x0ff) << 8);
	}

}