/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
public class Filter
{
	double[] y_effects;
	double[] x_effects;

	public Filter(Polynomial poles, Polynomial zeroes)
	{
		y_effects = translatePolynomial(poles);
		x_effects = translatePolynomial(zeroes);
	}

	double[] translatePolynomial(Polynomial p)
	{
		// find max delay.  Assume causal.
		int delay = 0;
		for (int i=0;i<p.getSize();i++)
		{
			SimplePolyTerm spt = p.getTerm(i);
			int power = spt.getPower();
			if (power < delay)
				delay = power;
		}
		double[] result = new double[(-1 * delay) + 1];
		for (int i=0;i<p.getSize();i++)
		{
			SimplePolyTerm spt = p.getTerm(i);
			int power = spt.getPower();
			double coeff = spt.getCoefficient();
			result[-1 * power] = coeff;
		}
		return result;		
	}

	public double[] run(double[] data)
	{
		double[] result = new double[data.length];
		for (int i=0;i<data.length;i++)
		{
			double temp = 0.0;
			// first apply zeroes (the FIR portion)
			for (int j=0;j<x_effects.length;j++)
			{
				if ((i-j) >= 0)
					temp = temp + (x_effects[j] * data[i-j]);
			}
			// now apply poles (the IIR portion).  Note that we are 
			// going to subtract, and we start from 1.
			for (int j=1;j<y_effects.length;j++)
			{
				if ((i-j) >= 0)
					temp = temp - (y_effects[j] * result[i-j]);
			}
			result[i] = temp;
		}
		return result;
	}
}