/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
/**
 *
 * Filter Computation Applet
 * 
 * By selecting poles and zeroes on a polar coordinate plane, you can create
 * filters of any length.  The fourier transform of the resulting filter will
 * also appear.
 *
 * In addition, you can alsp specify how many bits should be in the coefficient
 * for use with a hardware MAC.  The program will figure out what precision it
 * should use in its output coefficients and recalculate the 'applied' fourier
 * transform, plus another graph showing the error from the precision.
 *
 * KLA
 * Oct 5, 2004
 *
 */


import java.awt.*;
import java.applet.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class FilterCalculator extends Applet
                           implements MouseListener, MouseMotionListener, UnitCircleListener
{
  static final int WINDOWSIZE_X = 950;
  static final int WINDOWSIZE_Y = 820;

  static final int BUTTON_WIDTH = 80;
  static final int BUTTON_HEIGHT = 25;

  TextField orderField;

  UnitCircle uc;
  
  TextField[] z_coefficientFields = null;
  Label[] z_coefficientLabels = null;
  double[] z_coefficients = null;
  Label[] k_labels = null;
  Label passLabel = null;

  Label bitLengthLabel;
  TextField bitLengthField;


  double[] transform1;
  double[] transform2;
  Graph gd;
  Graph gf;
  Graph gferr;
  Graph errorGraph;

  TextArea zeroArea;

  public FilterCalculator() throws Exception
  {
    setLayout(null);
    setVisible(false);
    setSize(WINDOWSIZE_X, WINDOWSIZE_Y);
//    setLocation(10, 10);
    setBackground(Color.lightGray);
//    setResizable(false);
//    setTitle("Filter Calculator");
		
    addMouseListener(this);
    addMouseMotionListener(this);
    
    uc = new UnitCircle(200);
    uc.setBounds(50,50,uc.getWidth(),uc.getHeight());
    uc.setDataColor(Color.red);
    add(uc);
    uc.setVisible(true);
    uc.setListener(this);


    gf = new Graph(200,200);
    gf.setDataRegion(-1 * Math.PI, 2.0 * Math.PI / 200.0);
    gf.originIsAtBottom(true);
    gf.setBounds(500,50,gf.getWidth(),gf.getHeight());
    add(gf);
    gf.setVisible(true);


    Label label1 = new Label("Ideal Transform");
    label1.setBounds(500, 30, 200, 20);
    add(label1);
    label1.setVisible(true);

    bitLengthLabel = new Label("Bit Length: ");
    bitLengthLabel.setBounds(200, 300, 100, 30);
    add(bitLengthLabel);
    bitLengthLabel.setVisible(true);

    bitLengthField = new TextField("16");
    bitLengthField.setBounds(400, 300, 100, 30);
    add(bitLengthField);
    bitLengthField.setVisible(true);

    gferr = new Graph(200,200);
    gferr.setDataRegion(-1 * Math.PI, 2.0 * Math.PI / 200.0);
    gferr.originIsAtBottom(true);
    gferr.setBounds(50,400,gferr.getWidth(),gferr.getHeight());
    add(gferr);
    gferr.setVisible(true);

    Label label2 = new Label("Actual Transform");
    label2.setBounds(50, 380, 200, 20);
    add(label2);
    label2.setVisible(true);

    errorGraph = new Graph(200,200);
    errorGraph.setDataRegion(-1 * Math.PI, 2.0 * Math.PI / 200.0);
    errorGraph.originIsAtBottom(true);
    errorGraph.setBounds(500,400,gferr.getWidth(),gferr.getHeight());
    add(errorGraph);
    errorGraph.setVisible(true);

    Label label3 = new Label("Error in Implementation (diff from 0)");
    label3.setBounds(500, 380, 200, 20);
    add(label3);
    label3.setVisible(true);

    zeroArea  = new TextArea();
    zeroArea.setBounds(100, 660, WINDOWSIZE_X - 200, WINDOWSIZE_Y - 670);
    zeroArea.setVisible(true);
    add(zeroArea);


    setVisible(true);

    repaint();

  }

/*  public static void main(String[] args) throws Exception
  {
     FilterCalculator fc = new FilterCalculator();
  }
*/

	// applet interface method...do nothing?
	public void init()
	{
	}

	// applet interface method...do nothing?
	public void start()
	{
	}

	// applet interface method...do nothing?
	public void stop()
	{
	}

	// applet interface method...do nothing?
	public void destroy()
	{
	}


  public void mouseClicked(MouseEvent me)
  {
  }

  public void mouseEntered(MouseEvent me)
  {
    //System.out.println("ENTERED");
  }

  public void mouseExited(MouseEvent me)
  {
    //System.out.println("EXITED");
  }

  public void mousePressed(MouseEvent me)
  {
    //System.out.println("PRESSED");
  }

  public void mouseReleased(MouseEvent me)
  {
    //System.out.println("RELEASED");
  }

  public void mouseDragged(MouseEvent me)
  {
  }
	
  public void mouseMoved(MouseEvent me)
  {
  }


  static double[] impulseData = new double[200];
  static
  {
    impulseData[0] = 1.0;
  }


  public void notifyOnUpdate()
  {
    System.out.println("Unit Circle Updated");
    Polynomial poles = uc.getPoles();
    Polynomial zeroes = uc.getZeroes();
      System.out.println("Poles: "+poles);
      System.out.println("Zeroes: "+zeroes);
    Filter filter = new Filter(poles, zeroes);
    double[] result = filter.run(impulseData);
    double[] transform = FourierTransform.performTransform(result, 200);
    gf.setData(transform, 0, transform.length);
    gf.repaint();

    int bits = Integer.parseInt(bitLengthField.getText());
    // take off one bit for the sign
    bits--;
    // first determine the maximum magnitude in the poles and zeroes
    double maxmagnitude = 0.0;
    for (int i=0;i<poles.getSize();i++)
    {
      SimplePolyTerm spt = poles.getTerm(i);
      double mag = spt.getCoefficient();
      if (mag < 0)
        mag = -1.0 * mag;
      if (mag > maxmagnitude)
        maxmagnitude = mag;
    }

    for (int i=0;i<zeroes.getSize();i++)
    {
      SimplePolyTerm spt = zeroes.getTerm(i);
      double mag = spt.getCoefficient();
      if (mag < 0)
        mag = -1.0 * mag;
      if (mag > maxmagnitude)
        maxmagnitude = mag;
    }

    System.out.println("Max Magnitude = "+maxmagnitude);
    int bits_magnitude = 1;
    double compareval = 1.0;
    while (maxmagnitude > compareval)
    {
      bits_magnitude++;
      compareval = compareval * 2;
    }
    System.out.println("Bits required for magnitude: "+bits_magnitude);

    // can't do anything if the bit length ain't right
    if (bits_magnitude > bits)
      return;

    int bits_belowradix = bits - bits_magnitude;
    double multiplier = Math.pow(2.0, bits_belowradix);
    Polynomial newpoles = new Polynomial();
    Polynomial newzeroes = new Polynomial();
int shiftamount = 15 - bits_magnitude;
System.out.println("Poles: ");
System.out.print("\r\n dc16 "+shiftamount+", "+poles.getSize()+", ");
    for (int i=0;i<poles.getSize();i++)
    {
      SimplePolyTerm spt = poles.getTerm(i);
      double value = spt.getCoefficient();
      value = value * multiplier;
      int temp = (int) value;
System.out.print("0x"+Integer.toHexString(temp & 0x0ffff));
if ((i & 0x07)!=0x07)
{
    if (i!=(poles.getSize()-1))
        System.out.print(", ");
}
else if (i!=(poles.getSize()-1))
    System.out.print("\r\n dc16 ");
      value = (double)temp / multiplier;
      newpoles.add(new SimplePolyTerm(value, spt.getPower()));
    }

zeroArea.setText("");
zeroArea.append("\r\n");
zeroArea.append("Zeroes: \r\n");
zeroArea.append("\r\n dc16 ");
zeroArea.append("\r\n dc16 "+shiftamount+", "+zeroes.getSize()+", ");
System.out.println();
System.out.println("Zeroes: ");
System.out.print("\r\n dc16 ");
System.out.print("\r\n dc16 "+shiftamount+", "+zeroes.getSize()+", ");
    for (int i=0;i<zeroes.getSize();i++)
    {
      SimplePolyTerm spt = zeroes.getTerm(i);
      double value = spt.getCoefficient();
      value = value * multiplier;
      int temp = (int) value;
System.out.print("0x"+Integer.toHexString(temp & 0x0ffff));
zeroArea.append("0x"+Integer.toHexString(temp & 0x0ffff));
if ((i & 0x07)!=0x07)
{
    if (i!=(zeroes.getSize()-1))
    {
        System.out.print(", ");
        zeroArea.append(", ");
    }
}
else if (i!=(zeroes.getSize()-1))
    System.out.print("\r\n dc16 ");
      value = (double)temp / multiplier;
      newzeroes.add(new SimplePolyTerm(value, spt.getPower()));
    }
System.out.println();
System.out.println("Shift amount: "+(15 - bits_magnitude));
zeroArea.append("\r\n");
zeroArea.append("Shift amount: "+(15 - bits_magnitude));
      System.out.println("New Poles: "+newpoles);
      System.out.println("New Zeroes: "+newzeroes);
System.out.println("Multiplier: "+multiplier);    


    filter = new Filter(newpoles, newzeroes);
    result = filter.run(impulseData);
    double[] transform2 = FourierTransform.performTransform(result, 200);
    gferr.setData(transform2, 0, transform2.length);
    gferr.repaint();

    double[] error = new double[transform2.length];
    for (int i=0;i<error.length;i++)
    {
      error[i] = transform2[i] / transform[i];
    }

    errorGraph.setData(error, 0, error.length);
    errorGraph.repaint();

  }


  String formatDouble(double x)
  {
    String s = Double.toString(x);
    int index = s.indexOf('.');
    if (index==-1)
      return s;
    if ((index + 4) > s.length())
      return s;
    return s.substring(0, index+4);
  }

  double[] computeDifference(double[] t1, double[] t2)
  {
    int length = t1.length;
    if (length > t2.length)
        length = t2.length;
    double[] diff = new double[length];
    for (int i=0;i<length;i++)
    {
        diff[i] = t2[i] / t1[i];
    }
    return diff;
  }


}
