/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
public class FourierTransform
{


	public static double[] performTransform(double[] data, int num_w)
	{
		// X(w) = sum( x(n) * e^(-jwn))
		Polar[] fourierpoints = new Polar[data.length];
		for (int i=0;i<data.length;i++)
		{
			fourierpoints[i] = new Polar(data[i], -1 * i);
		}

		double[] transform = new double[num_w];
		double w_step = (2.0 * Math.PI)/(double)num_w;
		double w = -1 * Math.PI;
		int lastindex = num_w-1;
//System.out.print("start");
		for (int i=0;i<=num_w/2;i++)
		{
			double real = 0.0;
			double imag = 0.0;
			for (int j=0;j<fourierpoints.length;j++)
			{
				Polar p = fourierpoints[j];
				double angle = p.theta * w;
				real += p.radius * Math.cos(angle);
				imag += p.radius * Math.sin(angle);
//System.out.print("["+real+","+imag+","+angle+","+i+"]");
			}
//System.out.print(".");
			w += w_step;
			transform[i] = Math.sqrt(real*real + imag*imag);
//System.out.print(transform[i]);
			transform[lastindex] = transform[i];
			lastindex--;
		}
//System.out.println("end");
		return transform;
	}
}