/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
import java.util.*;

public class Polynomial
{
	Vector terms;

	public Polynomial()
	{
		terms = new Vector();
	}

	public void add(SimplePolyTerm spt)
	{
		boolean found = false;
		for (int i=0;i<terms.size();i++)
		{
			SimplePolyTerm term = (SimplePolyTerm)terms.elementAt(i);
			if (term.getPower() == spt.getPower())
			{
				found = true;
				term.add(spt);
			}
		}
		if (!found)
			terms.addElement(spt.clone());
	}

	public Polynomial multiply(Polynomial poly)
	{
		Polynomial newPoly = new Polynomial();
		for (int i=0;i<terms.size();i++)
		{
			for (int j=0;j<poly.terms.size();j++)
			{
				SimplePolyTerm term1 = (SimplePolyTerm) terms.elementAt(i);
				SimplePolyTerm term2 = (SimplePolyTerm) poly.terms.elementAt(j);
				SimplePolyTerm spt = term1.multiply(term2);
				newPoly.add(spt);
			}
		}
		return newPoly;
	}

	public String toString()
	{
		String s = "";
		for (int i=0;i<terms.size();i++)
		{
			SimplePolyTerm spt = (SimplePolyTerm) terms.elementAt(i);
			if (i!=0)
			{
				s += " ";
			}
			s += spt.getCoefficient();
			s += "x^(";
			s += spt.getPower();
			s+=")";
		}
		return s;
	}

	public int getSize()
	{
		return terms.size();
	}

	public SimplePolyTerm getTerm(int i)
	{
		return (SimplePolyTerm)terms.elementAt(i);
	}
}