/*---------------------------------------------------------------------------
 * Copyright (C) 2005 Dallas Semiconductor Corporation, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Dallas Semiconductor
 * shall not be used except as stated in the Dallas Semiconductor
 * Branding Policy.
 *---------------------------------------------------------------------------
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class UnitCircle extends Container
                           implements MouseListener, MouseMotionListener, ActionListener, TextListener
{
	Vector zeroes = null;
	Vector poles = null;
	boolean continuous = true;
	double start = 0;
	double step = 1;
	double max = 0.0;
	Color datacolor;

	int CIRCLE_DIAMETER;
	int GRAPH_SIZE;
	int PLOT_WIDTH;
	int radius;

	Button clearButton;
	Label radiuslabel;
	Label thetalabel;
	Label gainlabel;
	TextField gainfield;

	static final int BUFFER = 5;
	static final int DELTA = 6;

	UnitCircleListener listener = null; 

	public UnitCircle(int diameter)
	{
		CIRCLE_DIAMETER = diameter;
		PLOT_WIDTH = diameter + 200;
		setSize(CIRCLE_DIAMETER, PLOT_WIDTH);
		radius = 3 * CIRCLE_DIAMETER / 8;
		GRAPH_SIZE = diameter - 1;
		zeroes = new Vector();
		poles = new Vector();

		clearButton = new Button("Clear");
		clearButton.addActionListener(this);
		clearButton.setBounds(CIRCLE_DIAMETER+10, CIRCLE_DIAMETER/2 - 60, PLOT_WIDTH - CIRCLE_DIAMETER - 10, 30);
		add(clearButton);
		clearButton.setVisible(true);


		radiuslabel = new Label("Radius");
		radiuslabel.setBounds(CIRCLE_DIAMETER+10,  CIRCLE_DIAMETER/2 - 30, PLOT_WIDTH - CIRCLE_DIAMETER - 10, 30);
		add(radiuslabel);
		radiuslabel.setVisible(true);

		thetalabel = new Label("Theta");
		thetalabel.setBounds(CIRCLE_DIAMETER+10,  CIRCLE_DIAMETER/2, PLOT_WIDTH - CIRCLE_DIAMETER - 10, 30);
		add(thetalabel);
		thetalabel.setVisible(true);

		gainlabel = new Label("Gain: ");
		gainlabel.setBounds(CIRCLE_DIAMETER+10,  CIRCLE_DIAMETER/2 + 30, 40, 30);
		add(gainlabel);
		gainlabel.setVisible(true);

		gainfield= new TextField("1.0");
		gainfield.addTextListener(this);
		gainfield.setBounds(CIRCLE_DIAMETER+50,  CIRCLE_DIAMETER/2 + 30, PLOT_WIDTH - CIRCLE_DIAMETER - 50, 30);
		add(gainfield);
		gainfield.setVisible(true);

		addMouseListener(this);
		addMouseMotionListener(this);		
	}

	public double getGain()
	{
		try
		{
			return Double.valueOf(gainfield.getText()).doubleValue();
		}
		catch(Exception e)
		{
			gainfield.setText("1.0");
		}
		return 1.0;
	}

	public void setListener(UnitCircleListener ucl)
	{
		listener = ucl;
	}

	public int getHeight()
	{
		return CIRCLE_DIAMETER;
	}
	
	public int getWidth()
	{
		return PLOT_WIDTH;
	}

	public void setDataColor(Color c)
	{
		datacolor = c;
	}

	public void clear()
	{
		zeroes = new Vector();
		poles = new Vector();
		repaint();
	}

	public Polynomial getZeroes()
	{
		Vector tempz = new Vector();
		Polynomial poly = new Polynomial();
		double gain = getGain();
		SimplePolyTerm spt = new SimplePolyTerm(1.0, 0);
		poly.add(spt);
		while (zeroes.size() > 0)
		{
			Polar p1 = (Polar)zeroes.elementAt(0);
			zeroes.removeElementAt(0);
			Polar p2 = p1.getPolePair();
			if (p2 != null)
				zeroes.removeElement(p2);
			Polynomial temp = new Polynomial();
			temp.add(new SimplePolyTerm(1.0,0));
			tempz.addElement(p1);
			if (p2==null)
			{
				temp.add(new SimplePolyTerm(-p1.radius,-1));
			}
			else
			{
				tempz.addElement(p2);
				temp.add(new SimplePolyTerm(-2 * p1.radius * Math.cos(p1.theta), -1));
				temp.add(new SimplePolyTerm(p1.radius * p1.radius, -2));
			}
			poly = poly.multiply(temp);		
		}
		
		zeroes = tempz;
		Polynomial gainpoly = new Polynomial();
		gainpoly.add(new SimplePolyTerm(gain,0));
		return poly.multiply(gainpoly);
	}

	public Polynomial getPoles()
	{
		Vector tempp = new Vector();
		Polynomial poly = new Polynomial();
		double gain = getGain();
		SimplePolyTerm spt = new SimplePolyTerm(1.0, 0);
		poly.add(spt);
		while (poles.size() > 0)
		{
			Polar p1 = (Polar)poles.elementAt(0);
			poles.removeElementAt(0);
			Polar p2 = p1.getPolePair();
			if (p2 != null)
				poles.removeElement(p2);
			Polynomial temp = new Polynomial();
			temp.add(new SimplePolyTerm(1.0,0));
			tempp.addElement(p1);
			if (p2==null)
			{
				temp.add(new SimplePolyTerm(-p1.radius,-1));
			}
			else
			{
				tempp.addElement(p2);
				temp.add(new SimplePolyTerm(-2 * p1.radius * Math.cos(p1.theta), -1));
				temp.add(new SimplePolyTerm(p1.radius * p1.radius, -2));
			}
			poly = poly.multiply(temp);		
		}
		
		poles = tempp;
		Polynomial gainpoly = new Polynomial();
		gainpoly.add(new SimplePolyTerm(gain,0));
		return poly.multiply(gainpoly);
	}

	public void paint(Graphics g)
	{
		// first draw a border
		g.drawLine(0, 0, GRAPH_SIZE, 0);
		g.drawLine(GRAPH_SIZE, 0, GRAPH_SIZE, GRAPH_SIZE);
		g.drawLine(GRAPH_SIZE, GRAPH_SIZE, 0, GRAPH_SIZE);
		g.drawLine(0, GRAPH_SIZE, 0, 0);

		// first draw the X and Y axis
		g.drawLine(0, CIRCLE_DIAMETER/2, CIRCLE_DIAMETER, CIRCLE_DIAMETER/2);
		g.drawLine(CIRCLE_DIAMETER/2, 0, CIRCLE_DIAMETER/2, CIRCLE_DIAMETER);

		g.drawOval(CIRCLE_DIAMETER/8, CIRCLE_DIAMETER/8, 3*CIRCLE_DIAMETER/4, 3*CIRCLE_DIAMETER/4);
		
		g.setColor(datacolor);

		for (int i=0;i<zeroes.size();i++)
		{
			Polar p = (Polar)zeroes.elementAt(i);
			int x = (int)(radius * (p.radius * Math.cos(p.theta)));
			int y = (int)(radius * (p.radius * Math.sin(p.theta)));
			System.out.println(" Zero at ("+x+":"+y+")");
			x = GRAPH_SIZE/2 + x;
			y = GRAPH_SIZE/2 - y;
			g.drawOval(x-(DELTA/2), y-(DELTA/2), DELTA, DELTA);
		}

		for (int i=0;i<poles.size();i++)
		{
			Polar p = (Polar)poles.elementAt(i);
			int x = (int)(radius * (p.radius * Math.cos(p.theta)));
			int y = (int)(radius * (p.radius * Math.sin(p.theta)));
			x = GRAPH_SIZE/2 + x;
			y = GRAPH_SIZE/2 - y;
			System.out.println(" Pole at ("+x+":"+y+")");
			g.drawLine(x-(DELTA/2), y-(DELTA/2), x+(DELTA/2), y+(DELTA/2));
			g.drawLine(x-(DELTA/2), y+(DELTA/2), x+(DELTA/2), y-(DELTA/2));
		}


	}


  public void mouseClicked(MouseEvent me)
  {
	System.out.println("Clicked at "+me.getX() + " " + me.getY());
	int x = me.getX() - GRAPH_SIZE/2;
	int y = -1 * (me.getY() - GRAPH_SIZE/2);

	double r = Math.sqrt(x*x + y*y);
	r = r / (double)radius;

	double theta = Math.atan2(y, x);

	System.out.println("Clicked at ("+x+", "+y+"), or "+r+" at "+theta);
	Vector points = zeroes;
	Vector other = poles;

	int mod = me.getModifiers();
	if ((mod & MouseEvent.BUTTON3_MASK) != 0)
	{
		points = poles;
		other = zeroes;
	}

	// first check to see if it exists in the other.  If so, cancel them out.
	boolean found = false;
	for (int i=0;i<other.size();i++)
	{
		Polar p = (Polar)other.elementAt(i);
		if ((p.radius == r) && (p.theta == theta))
		{
			other.removeElementAt(i);
			if (theta != 0.0)
			{
				other.removeElement(p.getPolePair());
			}
			i--;
			found = true;
		}
	}

	if (!found)
	{
		Polar p1 = new Polar(r, theta);
		Polar p2 = new Polar(r, -theta);
		points.addElement(p1);
		if (theta != 0.0)
		{
			p1.setPolePair(p2);
			p2.setPolePair(p1);
			points.addElement(p2);
		}
	}
	repaint();
	if (listener != null)
		listener.notifyOnUpdate();
  }

  public void mouseEntered(MouseEvent me)
  {
    //System.out.println("ENTERED");
  }

  public void mouseExited(MouseEvent me)
  {
    //System.out.println("EXITED");
  }

  public void mousePressed(MouseEvent me)
  {
    //System.out.println("PRESSED");
  }

  public void mouseReleased(MouseEvent me)
  {
    //System.out.println("RELEASED");
  }

  public void mouseDragged(MouseEvent me)
  {
  }
	
  public void mouseMoved(MouseEvent me)
  {
	int x = me.getX();
	int y = me.getY();
	if (x > CIRCLE_DIAMETER) return;
	if (y > CIRCLE_DIAMETER) return;

	x = me.getX() - GRAPH_SIZE/2;
	y = -1 * (me.getY() - GRAPH_SIZE/2);

	double r = Math.sqrt(x*x + y*y);
	r = r / (double)radius;

	double theta = Math.atan2(y, x);

	radiuslabel.setText("Radius = "+r);
	thetalabel.setText("Theta = "+theta+" PI");

	

  }


	public void actionPerformed(ActionEvent ae)
	{
		Object o = ae.getSource();
		if (o==clearButton)
		{
			clear();
			if (listener != null)
				listener.notifyOnUpdate();
		}
	}


  public void textValueChanged(TextEvent te) 
  {
    Object o = te.getSource();
    if (o==gainfield)
    {
      System.out.println("Gain field changed");
      if (listener != null)
        listener.notifyOnUpdate();
    }
  }

}

