/* --------------------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        rtc
 *  Description:   Real Time Clock Convenience Functions
 *  Filename:      maxq2000_rtc.c
 *  Date:          October 13, 2004
 *  Version:       1.00
 *
 *  This file contains implementations for some basic convenience functions for 
 *  the RTC on the MAXQ2000.  It is not meant to be a complete library to support all 
 *  the functionality available on the MAXQ2000's real time clock.
 *
 *  See the file 'maxq2000_rtc.h' for documentation on function usage.
 *
 * --------------------------------------------------------------------------------------
 */
#include <iomaxq200x.h>
#include "maxq2000_rtc.h"

void rtc_init()
{
    RCNT = 0x0000;                        // turn on the 32kHz
    while (RCNT & 0x08) {}
    RCNT = 0x8000;                        // write enable the real-time clock enable bit
    RCNT = 0x8001;                        // start the real time clock
    RCNT = 0x0001;                        // turn off RTCE write access 
}

void rtc_setsecondsalarm(unsigned long alarmtime)
{
    while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
    RCNT = RCNT & 0xFFFD;                // disable the alarm time of day enable
    while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
    RASL = (int)alarmtime;
    RASH = (int)(alarmtime>>16);
    while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
    RCNT = RCNT | 0x02;                  // turn the alarm time of day back on
}

unsigned long rtc_getseconds()
{
    unsigned long seconds = RTSH;
    seconds = seconds << 16;
    seconds = seconds | RTSL;
    return seconds;
}

void rtc_setseconds(unsigned long time)
{
    while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RCNT = 0x8001;                       // set write enable bit
    while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RCNT = 0x8000;                       // set RTCE to clock NOT enabled
    while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RTSH = (time >> 16) & 0x0ffff;
    while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RTSL = time & 0x0ffff;
    while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RCNT = 0x8001;                       // turn the clock back on
}

void rtc_translate(unsigned long totalseconds, int* seconds, int* minutes, int* hours, int* days)
{
    unsigned long temp = totalseconds / SECONDS_PER_DAY;
    *days = (int)temp;
    totalseconds = totalseconds % SECONDS_PER_DAY;
    temp = totalseconds / SECONDS_PER_HOUR;
    *hours = (int)temp;
    totalseconds = totalseconds % SECONDS_PER_HOUR;
    temp = totalseconds / SECONDS_PER_MINUTE;
    *minutes = (int)temp;
    *seconds = (int)(totalseconds % SECONDS_PER_MINUTE);
}
