/* --------------------------------------------------------------------------------------
 *  Copyright (C) 2004 Dallas Semiconductor Corporation, All Rights Reserved.
 * 
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 * 
 *  The above copyright notice and this permission notice shall be included
 *  in all copies or substantial portions of the Software.
 * 
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 *  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES
 *  OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 *  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 *  OTHER DEALINGS IN THE SOFTWARE.
 * 
 *  Except as contained in this notice, the name of Dallas Semiconductor
 *  shall not be used except as stated in the Dallas Semiconductor
 *  Branding Policy.
 *
 *  Module:        lcd
 *  Description:   LCD Controller Convenience Functions
 *  Filename:      maxq2000ev_lcd.c
 *  Date:          October 13, 2004
 *  Version:       1.00
 *
 *  This file contains implementations for some basic convenience functions for 
 *  the LCD controller on the MAXQ2000 Evaluation Kit.  It is not meant to be a complete 
 *  library to support all the functionality available on the MAXQ2000's real time clock.
 *  This library will not work without modification for LCD screens and hardware 
 *  configurations other than those that come with the MAXQ2000.
 *
 *  See the file 'maxq2000ev_lcd.h' for documentation on function usage.
 *
 * --------------------------------------------------------------------------------------
 */
#include <iomaxq200x.h>
#include "maxq2000ev_lcd.h"


int PATTERNS[] = { LCD_PATTERN_0, LCD_PATTERN_1, LCD_PATTERN_2, LCD_PATTERN_3, 
                   LCD_PATTERN_4, LCD_PATTERN_5, LCD_PATTERN_6, LCD_PATTERN_7, 
                   LCD_PATTERN_8, LCD_PATTERN_9, LCD_PATTERN_A, LCD_PATTERN_B,
                   LCD_PATTERN_C, LCD_PATTERN_D, LCD_PATTERN_E, LCD_PATTERN_F 
                 };

/******************************************************************************
 * Returns the value that will need to be placed in one of LCD0-LCD3 to display 
 * a digit 0-9.  No bounds checking is done here.  If you ask for a digit other
 * than 0-9, you will get a bogus display.
 */
int lcd_getlcddigit(int digit)
{
    return PATTERNS[digit];
}

void lcd_shownumber(int value)
{
   int thisdigit = 0;
   unsigned char firstnonzero = 0;
   
   if (value < 0)
   {
       if (value <= -10000)
       {
           LCD4 = 0xC0;
           firstnonzero = 1;
       }
       else
       {
           LCD4 = 0x80;
       }
       value = -1 * value;
   }
   else
   {
       if (value >= 10000)
       {
          LCD4 = 0x40;
          firstnonzero = 1;
       }
       else
       {
          LCD4 = 0;
       }
   }
   thisdigit = (value / 1000) % 10;
   if (!((thisdigit == 0) && (firstnonzero == 0)))
   {
       LCD3 = lcd_getlcddigit(thisdigit);
       firstnonzero = 1;
   }
   else
   {
       // make sure no segments are lit
       LCD3 = 0;
   }

   thisdigit = (value / 100) % 10;
   if (!((thisdigit == 0) && (firstnonzero == 0)))
   {
       LCD2 = lcd_getlcddigit(thisdigit);
       firstnonzero = 1;
   }
   else
   {
       // make sure no segments are lit
       LCD2 = 0;
   }

   thisdigit = (value / 10) % 10;
   if (!((thisdigit == 0) && (firstnonzero == 0)))
   {
       LCD1 = lcd_getlcddigit(thisdigit);
       firstnonzero = 1;
   }
   else
   {
       // make sure no segments are lit
       LCD1 = 0;
   }

   // we still want to display LCD0 even if the value is all 0's
   LCD0 = lcd_getlcddigit((value) % 10);
}

void lcd_shownumber0(int value)
{
   if (value < 0)
   {
       if (value <= -10000)
           LCD4 = 0xC0;
       else
           LCD4 = 0x80;
       value = -1 * value;
   }
   else
   {
       if (value >= 10000)
          LCD4 = 0x40;
       else
          LCD4 = 0;
   }
   LCD3 = lcd_getlcddigit((value / 1000) % 10);
   LCD2 = lcd_getlcddigit((value / 100) % 10);
   LCD1 = lcd_getlcddigit((value / 10) % 10);
   LCD0 = lcd_getlcddigit((value) % 10);
}


void lcd_showtime(int hours, int minutes)
{
   LCD3 = lcd_getlcddigit((hours / 10) % 10);
   LCD2 = lcd_getlcddigit(hours % 10);
   LCD1 = lcd_getlcddigit((minutes / 10) % 10);
   LCD0 = lcd_getlcddigit(minutes % 10);
   LCD4 = 0x10;                // lights the middle 'colon' segment
}


void lcd_init()
{
  //LCRA_bit.FRM = 7;   // Set up frame frequency.
  //LCRA_bit.LCCS = 1;  // Set clock source to HF/128.
  //LCRA_bit.DUTY = 0;  // Set up static duty cycle.
  //LCRA_bit.LRA = 0;   // Set R-adj to 0.
  //LCRA_bit.LRIGC = 1; // Select external LCD drive power.
  LCRA = 0x03E0;        // Just do them all at once...

  LCFG_bit.PCF = 0x0F;// Set up all segments as outputs.
  LCFG_bit.OPM = 1;   // Set to normal operation mode.
  LCFG_bit.DPE = 1;   // Enable display.
}
